'use strict'

var request = require('supertest');

// Here we get hold of the express application 
// by using the exported 'getApp'-property
var app = require("../src/getApp").getApp;

// Test the get method
describe('GET /users', function(){

  // Happy path
  it('Test it responds with json', function(done){

    // the request-object is the supertest top level api
    request(app)
      .get('/users')   // Make sure this is the same or you get "Error: expected "Content-Type" matching /json/, got "text/html; charset=utf-8""
      .set('Accept', 'application/json')
      .auth('username', 'password')   // Send the user and password on the request
      .expect('Content-Type', /json/)
      .expect(200, done); // note that we're passing the done as parameter to the expect
  });

  // Unhappy path
  it('Test Get /crap fails with 404', function(done){

    // the request-object is the supertest top level api - https://www.npmjs.com/package/supertest for examples
    // you can pass done straight to any of the .expect() calls
    request(app)
      .get('/crap')   // Make sure this is right or you get "Error: expected "Content-Type" matching /json/, got "text/html; charset=utf-8""
      .expect(404) 
      .end(function(err, res) {   // catch and log the error

          // Must rethrow or test will pass!
          //if (err) throw err;
          // Or mocha version:
          if (err) {
              // Log the error
              console.log('err=' + err);

              // rethrow
              return done(err);  // mocha version
          }

          // Tell mocha all done
          done();
      });

  });

  // Unhappy path
  it('Test Get fails wrong login', function(done){

    // We need to set the user and password on our request to the server
    //.auth('username', 'passwordXXX')   // from superAgent.  Basic auth.  https://stackoverflow.com/questions/28756543/setting-basic-auth-in-mocha-and-supertest
    // Or: .set("Authorization", "basic " + new Buffer("username:password").toString("base64"))  // basic auth

    // the request-object is the supertest top level api - https://www.npmjs.com/package/supertest for examples
    // you can pass done straight to any of the .expect() calls
    // Info: https://github.com/visionmedia/supertest
    request(app)
      .get('/users')
      .auth('username', 'passwordXXX')   // Send the user and wrong password on the request
      .expect(401, "Get lost") 
      .end(function(err, res) {   // catch and log the error

          // Must rethrow or test will pass!
          //if (err) throw err;
          // Or mocha version:
          if (err) {
              // Log the error
              console.log('err=' + err);

              // rethrow
              return done(err);  // mocha version
          }

          // Tell mocha all done
          done();
      });

  });

});
